import 'dart:collection' show Queue;
import 'dart:io' show HttpClient;

import 'package:jetlog/jetlog.dart' show Handler, Record;
import 'package:jetlog/formatters.dart' show Formatter;

/// An HTTP handler sends logging records to the target server.
class HttpHandler<T> extends Handler {
  HttpHandler(this.url, {Duration idleTimeout})
      : _client = HttpClient(),
        _buffer = Queue(),
        super() {
    if (idleTimeout != null) _client.idleTimeout = idleTimeout;
  }

  final HttpClient _client;
  final Queue<Record> _buffer;
  Formatter encoder;

  /// An [url] to which records should be pushed.
  final Uri url;

  Future<void> _send() async {
    final record = _buffer.removeFirst();
    final result = encoder.call(record);

    try {
      final req = await _client.postUrl(url)
        ..write(result);

      return req.close();
    } catch (err) {
      _buffer.addFirst(record);
    }

    return _send();
  }

  void _add(Record record) => _buffer.add(record);

  @override
  void handle(Record record) {
    _add(record);
    _send();
  }

  @override
  Future<void> close() async {
    await super.close();

    return _client.close();
  }
}
